/* 8390.c: A general NS8390 ethernet driver core for linux. */
/*
	Written 1992-94 by Donald Becker.
  
	Copyright 1993 United States Government as represented by the
	Director, National Security Agency.

	This software may be used and distributed according to the terms
	of the GNU Public License, incorporated herein by reference.

	The author may be reached as becker@CESDIS.gsfc.nasa.gov, or C/O
	Center of Excellence in Space Data and Information Sciences
	   Code 930.5, Goddard Space Flight Center, Greenbelt MD 20771
  
  This is the chip-specific code for many 8390-based ethernet adaptors.
  This is not a complete driver, it must be combined with board-specific
  code such as ne.c, wd.c, 3c503.c, etc.

  Seeing how at least eight drivers use this code, (not counting the
  PCMCIA ones either) it is easy to break some card by what seems like
  a simple innocent change. Please contact me or Donald if you think
  you have found something that needs changing. -- PG


  Changelog:

  Paul Gortmaker	: remove set_bit lock, other cleanups.
  Paul Gortmaker	: add ei_get_8390_hdr() so we can pass skb's to 
			  ei_block_input() for eth_io_copy_and_sum().
  Paul Gortmaker	: exchange static int ei_pingpong for a #define,
			  also add better Tx error handling.
  Paul Gortmaker	: rewrite Rx overrun handling as per NS specs.
  Alexey Kuznetsov	: use the 8390's six bit hash multicast filter.
  Paul Gortmaker	: tweak ANK's above multicast changes a bit.
  Paul Gortmaker	: update packet statistics for v2.1.x


  Sources:
  The National Semiconductor LAN Databook, and the 3Com 3c503 databook.

  */

static const char *version =
    "8390.c:v1.10 9/23/94 Donald Becker (becker@cesdis.gsfc.nasa.gov)\n";

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/fs.h>
#include <linux/types.h>
#include <linux/ptrace.h>
#include <linux/string.h>
#include <asm/system.h>
#include <asm/uaccess.h>
#include <asm/bitops.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/delay.h>
#include <linux/errno.h>
#include <linux/fcntl.h>
#include <linux/in.h>
#include <linux/interrupt.h>
#include <linux/init.h>

#include <linux/netdevice.h>
#include <linux/etherdevice.h>
#ifdef CONFIG_NET_FASTROUTE
#include <linux/if_arp.h>
#include <net/ip.h>
#endif

#include "8390.h"

/* These are the operational function interfaces to board-specific
   routines.
	void reset_8390(struct device *dev)
		Resets the board associated with DEV, including a hardware reset of
		the 8390.  This is only called when there is a transmit timeout, and
		it is always followed by 8390_init().
	void block_output(struct device *dev, int count, const unsigned char *buf,
					  int start_page)
		Write the COUNT bytes of BUF to the packet buffer at START_PAGE.  The
		"page" value uses the 8390's 256-byte pages.
	void get_8390_hdr(struct device *dev, struct e8390_hdr *hdr, int ring_page)
		Read the 4 byte, page aligned 8390 header. *If* there is a
		subsequent read, it will be of the rest of the packet.
	void block_input(struct device *dev, int count, struct sk_buff *skb, int ring_offset)
		Read COUNT bytes from the packet buffer into the skb data area. Start 
		reading from RING_OFFSET, the address as the 8390 sees it.  This will always
		follow the read of the 8390 header. 
*/
#define ei_reset_8390 (ei_local->reset_8390)
#define ei_block_output (ei_local->block_output)
#define ei_block_input (ei_local->block_input)
#define ei_get_8390_hdr (ei_local->get_8390_hdr)

/* use 0 for production, 1 for verification, >2 for debug */
#ifdef EI_DEBUG
int ei_debug = EI_DEBUG;
#else
int ei_debug = 1;
#endif

/* Index to functions. */
static void ei_tx_intr(struct device *dev);
static void ei_tx_err(struct device *dev);
static void ei_receive(struct device *dev, int overrun);
static void ei_rx_overrun(struct device *dev);

/* Routines generic to NS8390-based boards. */
static void NS8390_trigger_send(struct device *dev, unsigned int length,
								int start_page);
static void set_multicast_list(struct device *dev);

static void e8390_xon(struct device *dev);

/* Open/initialize the board.  This routine goes all-out, setting everything
   up anew at each open, even though many of these registers should only
   need to be set once at boot.
   */
int ei_open(struct device *dev)
{
    struct ei_device *ei_local = (struct ei_device *) dev->priv;

    /* This can't happen unless somebody forgot to call ethdev_init(). */
    if (ei_local == NULL) {
	printk(KERN_EMERG "%s: ei_open passed a non-existent device!\n", dev->name);
	return -ENXIO;
    }
    
    ei_reset_8390(dev);
    NS8390_init(dev, 1);
    dev->start = 1;
#ifdef DEBUG_8390_IRQ
    ei_local->irqlock = 0;
#endif
    ei_local->last_rx = jiffies;
#ifdef CONFIG_NET_HW_FLOWCONTROL
    ei_local->fc_bit = netdev_register_fc(dev, e8390_xon);
#endif
#ifdef CONFIG_NET_FASTROUTE
    dev->tx_semaphore = 1;
#endif
    return 0;
}

/* Opposite of above. Only used when "ifconfig <devname> down" is done. */
int ei_close(struct device *dev)
{
    struct ei_device *ei_local = (struct ei_device *) dev->priv;
#ifdef CONFIG_NET_HW_FLOWCONTROL
    if (ei_local->fc_bit) {
	    int bit = ei_local->fc_bit;
	    ei_local->fc_bit = 0;
	    netdev_unregister_fc(bit);
    }
#endif
    NS8390_init(dev, 0);
    dev->start = 0;
#ifdef CONFIG_NET_FASTROUTE
    dev->tx_semaphore = 0;
    if (1) {
	    struct sk_buff *skb = xchg(&ei_local->skb_cache, NULL);
	    if (skb)
		    kfree_skb(skb);
    }
#endif
    return 0;
}

static int ei_start_xmit(struct sk_buff *skb, struct device *dev)
{
    int e8390_base = dev->base_addr;
    struct ei_device *ei_local = (struct ei_device *) dev->priv;
    int length, send_length, output_page;

/*
 *  We normally shouldn't be called if dev->tbusy is set, but the
 *  existing code does anyway. If it has been too long since the
 *  last Tx, we assume the board has died and kick it.
 */
 
#ifdef CONFIG_NET_FASTROUTE
    if (xchg(&dev->tx_semaphore, 0) == 0) {
	    sti();
	    if (dev->tbusy)
		    return 1;
	    printk("%s: sem down\n", dev->name);
	    dev_kfree_skb(skb);
	    return 0;
    }
#endif
    if (dev->tbusy) {	/* Do timeouts, just like the 8003 driver. */
		int txsr = inb(e8390_base+EN0_TSR), isr;
		int tickssofar = jiffies - dev->trans_start;
		sti();
		if (tickssofar < TX_TIMEOUT ||	(tickssofar < (TX_TIMEOUT+5) && ! (txsr & ENTSR_PTX))) {
#ifdef CONFIG_NET_FASTROUTE
			dev->tx_semaphore = 1;
#endif
			return 1;
		}
		ei_local->stat.tx_errors++;
		isr = inb(e8390_base+EN0_ISR);
		if (dev->start == 0) {
			printk("%s: xmit on stopped card\n", dev->name);
#ifdef CONFIG_NET_FASTROUTE
			dev->tx_semaphore = 1;
#endif
			dev_kfree_skb(skb);
			return 0;
		}

		/*
		 * Note that if the Tx posted a TX_ERR interrupt, then the
		 * error will have been handled from the interrupt handler
		 * and not here. Error statistics are handled there as well.
		 */

		printk(KERN_DEBUG "%s: Tx timed out, %s TSR=%#2x, ISR=%#2x, t=%d.\n",
		   dev->name, (txsr & ENTSR_ABT) ? "excess collisions." :
		   (isr) ? "lost interrupt?" : "cable problem?", txsr, isr, tickssofar);

		if (!isr && !ei_local->stat.tx_packets) {
		   /* The 8390 probably hasn't gotten on the cable yet. */
		   ei_local->interface_num ^= 1;   /* Try a different xcvr.  */
		}

		/* Try to restart the card.  Perhaps the user has fixed something. */
		ei_reset_8390(dev);
		NS8390_init(dev, 1);
		dev->trans_start = jiffies;
    }

    length = skb->len;

    /* Mask interrupts from the ethercard. */
    outb_p(0x00, e8390_base + EN0_IMR);
    if (dev->interrupt) {
	    sti();
	    printk("%s: Tx request while isr active.\n",dev->name);
	    outb_p(ENISR_ALL, e8390_base + EN0_IMR);
	    ei_local->stat.tx_errors++;
#ifdef CONFIG_NET_FASTROUTE
	    dev->tx_semaphore = 1;
#endif
	    dev_kfree_skb(skb);
	    return 0;
    }
    sti();
#ifdef DEBUG_8390_IRQ
    if (xchg(&ei_local->irqlock, 1)) {
	    /* Absolutely impossible. */
	    printk("%s: Tx request while irqlock!\n",dev->name);
	    ei_local->stat.tx_errors++;
#ifdef CONFIG_NET_FASTROUTE
	    dev->tx_semaphore = 1;
#endif
	    dev_kfree_skb(skb, FREE_WRITE);
	    return 0;
    }
#endif
    if (dev->tbusy) {
	    printk("%s : tbusy ", dev->name);
    }

    send_length = ETH_ZLEN < length ? length : ETH_ZLEN;
    
#ifdef EI_PINGPONG

    /*
     * We have two Tx slots available for use. Find the first free
     * slot, and then perform some sanity checks. With two Tx bufs,
     * you get very close to transmitting back-to-back packets. With
     * only one Tx buf, the transmitter sits idle while you reload the
     * card, leaving a substantial gap between each transmitted packet.
     */

    if (ei_local->tx1 == 0) {
	output_page = ei_local->tx_start_page;
	ei_local->tx1 = send_length;
    } else if (ei_local->tx2 == 0) {
	output_page = ei_local->tx_start_page + TX_1X_PAGES;
	ei_local->tx2 = send_length;
#ifdef TX3
    } else if (ei_local->tx3 == 0) {
	output_page = ei_local->tx_start_page + 2*TX_1X_PAGES;
	ei_local->tx3 = send_length;
#endif
    } else {	/* We should never get here. */
	if (ei_debug)
		printk("%s: No Tx buffers free! irq=%d tx1=%d tx2=%d last=%d\n",
			dev->name, dev->interrupt, ei_local->tx1, ei_local->tx2, ei_local->lasttx);
#ifdef DEBUG_8390_IRQ
	ei_local->irqlock = 0;
#endif
	dev->tbusy = 1;
	outb_p(ENISR_ALL, e8390_base + EN0_IMR);
	ei_local->stat.tx_errors++;
#ifdef CONFIG_NET_FASTROUTE
	dev->tx_semaphore = 1;
#endif
	return 1;
    }

    /*
     * Okay, now upload the packet and trigger a send if the transmitter
     * isn't already sending. If it is busy, the interrupt handler will
     * trigger the send later, upon receiving a Tx done interrupt.
     */

    ei_block_output(dev, length, skb->data, output_page);
    if (! ei_local->txing) {
	ei_local->txing = 1;
	NS8390_trigger_send(dev, send_length, output_page);
	dev->trans_start = jiffies;
	if (output_page == ei_local->tx_start_page) {
		ei_local->tx1 = -1;
		ei_local->lasttx = -1;
#ifdef TX3
	} else if (output_page == ei_local->tx_start_page + 2*TX_1X_PAGES) {
		ei_local->tx3 = -1;
		ei_local->lasttx = -3;
#endif
	} else {
		ei_local->tx2 = -1;
		ei_local->lasttx = -2;
	}
    } else
	ei_local->txqueue++;

#ifndef TX2
    dev->tbusy = (ei_local->tx1  &&  ei_local->tx2);
#else
    dev->tbusy = (ei_local->tx1  &&  ei_local->tx2 && ei_local->tx3);
#endif

#else	/* EI_PINGPONG */

    /*
     * Only one Tx buffer in use. You need two Tx bufs to come close to
     * back-to-back transmits. Expect a 20 -> 25% performance hit on
     * reasonable hardware if you only use one Tx buffer.
     */

    ei_block_output(dev, length, skb->data, ei_local->tx_start_page);
    ei_local->txing = 1;
    NS8390_trigger_send(dev, send_length, ei_local->tx_start_page);
    dev->trans_start = jiffies;
    dev->tbusy = 1;

#endif	/* EI_PINGPONG */

    /* Turn 8390 interrupts back on. */
#ifdef DEBUG_8390_IRQ
    ei_local->irqlock = 0;
#endif
    outb_p(ENISR_ALL, e8390_base + EN0_IMR);

#ifdef CONFIG_NET_FASTROUTE
    dev->tx_semaphore = 1;
#endif
    dev_kfree_skb (skb);
    ei_local->stat.tx_bytes += send_length;
    
    return 0;
}

#ifdef EI_PINGPONG
#ifdef CONFIG_NET_FASTROUTE
static int ei_x_xmit(__u8 *hdr, int len, struct device *dev,
		     struct device *idev, int isp)
{
    int e8390_base = dev->base_addr;
    struct ei_device *ei_local = (struct ei_device *) dev->priv;
    int output_page;

#ifdef CONFIG_NET_FASTROUTE
    if (xchg(&dev->tx_semaphore, 0) == 0) {
	    sti();
	    return 1;
    }
#endif

    /* Mask interrupts from the ethercard. */
    outb_p(0x00, e8390_base + EN0_IMR);
    if (dev->interrupt) {
	    sti();
	    printk("%s: xTx request while isr active.\n",dev->name);
	    outb_p(ENISR_ALL, e8390_base + EN0_IMR);
	    ei_local->stat.tx_errors++;
#ifdef CONFIG_NET_FASTROUTE
	    dev->tx_semaphore = 1;
#endif
	    return 0;
    }
    sti();

#ifdef DEBUG_8390_IRQ
    if (xchg(&ei_local->irqlock, 1)) {
	    /* Absolutely impossible. */
	    printk("%s: xTx request while irqlock!\n",dev->name);
	    ei_local->stat.tx_errors++;
#ifdef CONFIG_NET_FASTROUTE
	    dev->tx_semaphore = 1;
#endif
	    return 0;
    }
#endif
#ifdef E8390_LATENCY_REDUCTION
    if (dev->tbusy) {
	    /* Latency accelerator, seems it works */
	    if (inb_p(e8390_base) & E8390_TRANS) {
#ifdef DEBUG_8390_IRQ
		    ei_local->irqlock = 0;
#endif
		    outb_p(ENISR_ALL, e8390_base + EN0_IMR);
#ifdef CONFIG_NET_FASTROUTE
		    dev->tx_semaphore = 1;
#endif
		    return 1;
	    }
	    dev_fastroute_stat.latency_reduction++;
	    ei_tx_intr(dev);
    }
#endif

    /*
     * We have two Tx slots available for use. Find the first free
     * slot, and then perform some sanity checks. With two Tx bufs,
     * you get very close to transmitting back-to-back packets. With
     * only one Tx buf, the transmitter sits idle while you reload the
     * card, leaving a substantial gap between each transmitted packet.
     */

    if (ei_local->tx1 == 0) {
	output_page = ei_local->tx_start_page;
	ei_local->tx1 = len;
    } else if (ei_local->tx2 == 0) {
	output_page = ei_local->tx_start_page + TX_1X_PAGES;
	ei_local->tx2 = len;
#ifdef TX3
    } else if (ei_local->tx3 == 0) {
	output_page = ei_local->tx_start_page + TX_1X_PAGES*2;
	ei_local->tx3 = len;
#endif
    } else {	/* We should never get here. */
	if (ei_debug)
		printk("%s: No xTx buffers free! irq=%d tx1=%d tx2=%d last=%d\n",
		       dev->name, dev->interrupt, ei_local->tx1, ei_local->tx2, ei_local->lasttx);
#ifdef DEBUG_8390_IRQ
	ei_local->irqlock = 0;
#endif
	dev->tbusy = 1;
	outb_p(ENISR_ALL, e8390_base + EN0_IMR);
	ei_local->stat.tx_errors++;
#ifdef CONFIG_NET_FASTROUTE
	dev->tx_semaphore = 1;
#endif
	return 1;
    }

    ei_local->block_xio(idev, dev, hdr, isp, output_page, len);

    if (!ei_local->txing) {
	ei_local->txing = 1;
	NS8390_trigger_send(dev, len, output_page);
	dev->trans_start = jiffies;
	if (output_page == ei_local->tx_start_page) {
		ei_local->tx1 = -1;
		ei_local->lasttx = -1;
#ifdef TX3
	} else if (output_page == ei_local->tx_start_page + 2*TX_1X_PAGES) {
		ei_local->tx3 = -1;
		ei_local->lasttx = -3;
#endif
	} else {
		ei_local->tx2 = -1;
		ei_local->lasttx = -2;
	}
    } else
	ei_local->txqueue++;

#ifndef TX3
    dev->tbusy = (ei_local->tx1 && ei_local->tx2);
#else
    dev->tbusy = (ei_local->tx1 && ei_local->tx2 && ei_local->tx3);
#endif

    /* Turn 8390 interrupts back on. */
#ifdef DEBUG_8390_IRQ
    ei_local->irqlock = 0;
#endif
    outb_p(ENISR_ALL, e8390_base + EN0_IMR);

    ei_local->stat.tx_bytes += len;
    
#ifdef CONFIG_NET_FASTROUTE
    dev->tx_semaphore = 1;
#endif
    return 0;
}
#endif
#endif



/* The typical workload of the driver:
   Handle the ether interface interrupts. */
void ei_interrupt(int irq, void *dev_id, struct pt_regs * regs)
{
    struct device *dev = dev_id;
    int e8390_base = dev->base_addr;
    int interrupts;
    int serve = MAX_SERVICE;
    struct ei_device *ei_local = (struct ei_device *) dev->priv;

    if (xchg(&dev->interrupt, 1)) {
	    sti();
	    /* This might just be an interrupt for a PCI device sharing this line */
	    if (ei_debug)
		    printk("%s: Reentering the interrupt handler! isr=%#2x imr=%#2x.\n",
			   dev->name, inb_p(e8390_base + EN0_ISR),
			   inb_p(e8390_base + EN0_IMR));
	    return;
    }
#ifdef DEBUG_8390_IRQ
    if (ei_local->irqlock) {
	    dev->interrupt = 0;
	    sti();
	    /* It is IMPOSSIBLE */
	    printk("%s: Interrupted while interrupts are masked! isr=%#2x imr=%#2x.\n",
		   dev->name, inb_p(e8390_base + EN0_ISR),
		   inb_p(e8390_base + EN0_IMR));
	    return;
    }
#endif
    sti();

    /* Change to page 0 and read the intr status reg. */
    outb_p(E8390_NODMA+E8390_PAGE0, e8390_base + E8390_CMD);

    /* !!Assumption!! -- we stay in page 0.	 Don't break this. */
    while ((interrupts = inb_p(e8390_base + EN0_ISR)) != 0 && serve--) {
		if (dev->start == 0) {
			printk("%s: interrupt from stopped card\n", dev->name);
			interrupts = 0;
			break;
		}
#ifdef DELAY_DMA
		/* Ack RDC now, otherwise it will be stick in ISR */
		if (interrupts & ENISR_RDC) {
			if (xchg(&ei_local->dmaing, 0) == 0)
				printk("%s: unexp rdc\n", dev->name);
			outb_p(ENISR_RDC, e8390_base + EN0_ISR);
		}
#endif
		if (interrupts & ENISR_OVER) {
			ei_rx_overrun(dev);
		} else if (interrupts & (ENISR_RX+ENISR_RX_ERR)) {
			/* Got a good (?) packet. */
			ei_receive(dev, 0);
			ei_local->last_rx = jiffies;
		}
		/* Push the next to-transmit packet through. */
		if (interrupts & ENISR_TX) {
			ei_tx_intr(dev);
		} else if (interrupts & ENISR_TX_ERR) {
			ei_tx_err(dev);
		}

		if (interrupts & ENISR_COUNTERS) {
			ei_local->stat.rx_frame_errors += inb_p(e8390_base + EN0_COUNTER0);
			ei_local->stat.rx_crc_errors   += inb_p(e8390_base + EN0_COUNTER1);
			ei_local->stat.rx_missed_errors+= inb_p(e8390_base + EN0_COUNTER2);
			outb_p(ENISR_COUNTERS, e8390_base + EN0_ISR); /* Ack intr. */
		}

#ifndef DELAY_DMA
		/* Ignore any RDC interrupts that make it back to here. */
		if (interrupts & ENISR_RDC) {
			outb_p(ENISR_RDC, e8390_base + EN0_ISR);
			ei_local->dmaing = 0;
		}
#endif

		outb_p(E8390_NODMA+E8390_PAGE0+E8390_START, e8390_base + E8390_CMD);
    }

    if (interrupts && ei_debug) {
		outb_p(E8390_NODMA+E8390_PAGE0+E8390_START, e8390_base + E8390_CMD);
		printk("%s: Too much work at interrupt, status %#2.2x\n",
		       dev->name, interrupts);
		outb_p(ENISR_ALL, e8390_base + EN0_ISR); /* Ack. most intrs. */
    }
    if (jiffies - ei_local->last_rx > 60*HZ && dev->start) {
	    ei_local->last_rx = jiffies;
	    printk("%s: no packets for 1min\n", dev->name);
	    ei_reset_8390(dev);
	    NS8390_init(dev, 1);
    }
    dev->interrupt = 0;
    return;
}

/*
 * A transmitter error has happened. Most likely excess collisions (which
 * is a fairly normal condition). If the error is one where the Tx will
 * have been aborted, we try and send another one right away, instead of
 * letting the failed packet sit and collect dust in the Tx buffer. This
 * is a much better solution as it avoids kernel based Tx timeouts, and
 * an unnecessary card reset.
 */

static void ei_tx_err(struct device *dev)
{
    int e8390_base = dev->base_addr;
    unsigned char txsr = inb_p(e8390_base+EN0_TSR);
    unsigned char tx_was_aborted = txsr & (ENTSR_ABT+ENTSR_FU);
    struct ei_device *ei_local = (struct ei_device *) dev->priv;

#ifdef VERBOSE_ERROR_DUMP
    printk(KERN_DEBUG "%s: transmitter error (%#2x): ", dev->name, txsr);
    if (txsr & ENTSR_ABT)
		printk("excess-collisions ");
    if (txsr & ENTSR_ND)
		printk("non-deferral ");
    if (txsr & ENTSR_CRS)
		printk("lost-carrier ");
    if (txsr & ENTSR_FU)
		printk("FIFO-underrun ");
    if (txsr & ENTSR_CDH)
		printk("lost-heartbeat ");
    printk("\n");
#endif

    outb_p(ENISR_TX_ERR, e8390_base + EN0_ISR); /* Ack intr. */

    if (tx_was_aborted)
		ei_tx_intr(dev);
    else {
		ei_local->stat.tx_errors++;
		if (txsr & ENTSR_CRS) ei_local->stat.tx_carrier_errors++;
		if (txsr & ENTSR_CDH) ei_local->stat.tx_heartbeat_errors++;
		if (txsr & ENTSR_OWC) ei_local->stat.tx_window_errors++;
    }
}

/* We have finished a transmit: check for errors and then trigger the next
   packet to be sent. */
static void ei_tx_intr(struct device *dev)
{
    int e8390_base = dev->base_addr;
    int status = inb(e8390_base + EN0_TSR);
    struct ei_device *ei_local = (struct ei_device *) dev->priv;
    
    outb_p(ENISR_TX, e8390_base + EN0_ISR); /* Ack intr. */

#ifdef EI_PINGPONG

    /*
     * There are two Tx buffers, see which one finished, and trigger
     * the send of another one if it exists.
     */
    ei_local->txqueue--;
    if (ei_local->tx1 < 0) {
	ei_local->tx1 = 0;
	dev->tbusy = 0;
	if (ei_local->tx2 > 0) {
		ei_local->txing = 1;
		NS8390_trigger_send(dev, ei_local->tx2, ei_local->tx_start_page + 6);
		dev->trans_start = jiffies;
		ei_local->tx2 = -1,
		ei_local->lasttx = 2;
	} else
		ei_local->lasttx = 20, ei_local->txing = 0;
#ifndef TX3
    } else if (ei_local->tx2 < 0) {
	ei_local->tx2 = 0;
	dev->tbusy = 0;
	if (ei_local->tx1 > 0) {
		ei_local->txing = 1;
		NS8390_trigger_send(dev, ei_local->tx1, ei_local->tx_start_page);
		dev->trans_start = jiffies;
		ei_local->tx1 = -1;
		ei_local->lasttx = 1;
	} else
		ei_local->lasttx = 10, ei_local->txing = 0;
#else
    } else if (ei_local->tx2 < 0) {
	ei_local->tx2 = 0;
	dev->tbusy = 0;
	if (ei_local->tx3 > 0) {
		ei_local->txing = 1;
		NS8390_trigger_send(dev, ei_local->tx3, ei_local->tx_start_page+2*TX_1X_PAGES);
		dev->trans_start = jiffies;
		ei_local->tx3 = -1;
		ei_local->lasttx = 3;
	} else
		ei_local->lasttx = 10, ei_local->txing = 0;
    } else if (ei_local->tx3 < 0) {
	ei_local->tx3 = 0;
	dev->tbusy = 0;
	if (ei_local->tx1 > 0) {
		ei_local->txing = 1;
		NS8390_trigger_send(dev, ei_local->tx1, ei_local->tx_start_page);
		dev->trans_start = jiffies;
		ei_local->tx1 = -1;
		ei_local->lasttx = 1;
	} else
		ei_local->lasttx = 10, ei_local->txing = 0;
#endif
    }
#ifndef CONFIG_NET_FASTROUTE
    else
	printk("%s: unexpected TX-done interrupt, lasttx=%d.\n",
		   dev->name, ei_local->lasttx);
#endif

#else	/* EI_PINGPONG */
    /*
     *  Single Tx buffer: mark it free so another packet can be loaded.
     */
    ei_local->txing = 0;
    dev->tbusy = 0;
#endif

    /* Minimize Tx latency: update the statistics after we restart TXing. */
    if (status & ENTSR_COL)
	ei_local->stat.collisions++;
    if (status & ENTSR_PTX)
	ei_local->stat.tx_packets++;
    else {
	ei_local->stat.tx_errors++;
	if (status & ENTSR_ABT) {
		ei_local->stat.tx_aborted_errors++;
		ei_local->stat.collisions += 16;
	}
	if (status & ENTSR_CRS) ei_local->stat.tx_carrier_errors++;
	if (status & ENTSR_FU)  ei_local->stat.tx_fifo_errors++;
	if (status & ENTSR_CDH) ei_local->stat.tx_heartbeat_errors++;
	if (status & ENTSR_OWC) ei_local->stat.tx_window_errors++;
    }

    mark_bh (NET_BH);
}

#ifdef CONFIG_NET_HW_FLOWCONTROL
static void e8390_xon(struct device *dev)
{
    int e8390_base = dev->base_addr;
    struct ei_device *ei_local = (struct ei_device *) dev->priv;

	clear_bit(ei_local->fc_bit, &netdev_fc_xoff);
	if (dev->start)
		outb_p(ei_local->rxcfg, e8390_base + EN0_RXCR);
}
#endif

#ifdef CONFIG_NET_FASTROUTE
static int e8390_accept_fastpath(struct device *dev, struct dst_entry *dst)
{
	struct device *odev = dst->dev;

	if (dst->ops->protocol != __constant_htons(ETH_P_IP))
			return -1;
	if (odev->type != ARPHRD_ETHER || odev->accept_fastpath == NULL ||
	    (odev->hard_start_xmit == dev->hard_start_xmit &&
	     (((struct ei_device*)dev->priv)->block_xio == NULL ||
	      ((struct ei_device*)odev->priv)->block_xio == NULL)))
		return -1;

	return 0;
}

/*
   Return values:

   0 - packet has gone by fast path.
   1 - packet was going to go by fast path, but output device is busy (semifast)
   2 - fast path is hit, but packet is a bit strange. (NI)
   3 - fast path miss.
   4 - oom.
 */

static int e8390_fast_route(struct device *dev, struct sk_buff **skb_p,
			    int len, int isp)
{
	struct ei_device *ei_local = (struct ei_device *) dev->priv;
	struct device *odev;
	struct sk_buff *skb, *skb2;
	struct ethhdr *eth;
	struct rtable *rt;
	struct iphdr *iph;
	unsigned h;
	int xio_failure;

	/* Try to get skb from cache. Not that length does not matter,
	   at the first stage we need only ETH_ZLEN==60 bytes
	 */
	skb = ei_local->skb_cache;
	if (skb == NULL) {
		skb = dev_alloc_skb(len+2);
		if (skb == NULL)
			goto oom;
		skb_reserve(skb, 2);
		skb->dev = dev;
		ei_local->skb_cache = skb;
	}

	/* Get 60 bytes of the packet */
	ei_block_input(dev, 60, skb, (isp<<8) + 4);

	eth = (void*)skb->data;
	if (eth->h_proto != __constant_htons(ETH_P_IP))
		goto miss;

	iph = (struct iphdr*)(skb->data + ETH_HLEN);
	h = (*(u8*)&iph->daddr^*(u8*)&iph->saddr)&NETDEV_FASTROUTE_HMASK;
	rt = (struct rtable*)(dev->fastpath[h]);
	if (rt == NULL ||
	    ((u16*)&iph->daddr)[0] != ((u16*)&rt->key.dst)[0] ||
	    ((u16*)&iph->daddr)[1] != ((u16*)&rt->key.dst)[1] ||
	    ((u16*)&iph->saddr)[0] != ((u16*)&rt->key.src)[0] ||
	    ((u16*)&iph->saddr)[1] != ((u16*)&rt->key.src)[1] ||
	    rt->u.dst.obsolete)
		goto miss;

	odev = rt->u.dst.dev;
	dev_fastroute_stat.hits++;

	if (*(u8*)iph != 0x45 ||
	    (eth->h_dest[0]&1) ||
	    !neigh_is_valid(rt->u.dst.neighbour) ||
	    iph->ttl <= 1)
		goto suspect;

	ip_decrease_ttl(iph);
	ip_statistics.IpInReceives++;
	ip_statistics.IpForwDatagrams++;

	/* Could use hh cache */
	memcpy(eth->h_source, odev->dev_addr, 6);
	memcpy(eth->h_dest, rt->u.dst.neighbour->ha, 6);

	/* The best case: direct IO not requiring any temporary storage */

	xio_failure = 0;
	if (odev->hard_start_xmit == dev->hard_start_xmit) {
		cli();
		if (odev->tx_semaphore &&
#ifndef E8390_LATENCY_REDUCTION
		    odev->tbusy == 0 &&
#endif
		    odev->interrupt == 0 &&
		    ei_x_xmit(skb->data, len, odev, dev, isp) == 0) {
			dev_fastroute_stat.succeed++;
			return 0;
		}
		sti();
		xio_failure = 1;
	}

	/* Need to read packet to memory */
	skb2 = skb;
	if (len > skb_tailroom(skb)) {
		skb2 = dev_alloc_skb(len+2);
		if (skb2 == NULL)
			goto oom;
		skb_reserve(skb2, 2);
		memcpy(skb2->data, skb->data, 60);
	}
	skb2->dev = odev;
	skb_put(skb2, len);
	if (len > 60) {
		skb2->data += 60;
		ei_block_input(dev, len - 60, skb2, (isp<<8) + 60 + 4);
		skb2->data -= 60;
	}

	if (!xio_failure) {
		atomic_inc(&skb2->users);

		cli();
		if (odev->tx_semaphore &&
		    odev->tbusy == 0 &&
		    odev->interrupt == 0 &&
		    odev->hard_start_xmit(skb2, odev) == 0) {
			sti();
			dev_fastroute_stat.succeed++;
			if (atomic_read(&skb2->users) == 1) {
				/* Excellent! skb2 returned to us:
				   restore mangled data and record it in cache
				   */
				skb2->tail = skb2->data;
				skb2->len = 0;
				skb2->dev = dev;
				if (skb2 != skb) {
					/* New skb is better: it is larger and
					   it is more recent from viewpoint of cache.
					   */
					ei_local->skb_cache = skb2;
					kfree_skb(skb);
				}
			} else {
				kfree_skb(skb2);
				if (skb2 == skb)
					ei_local->skb_cache = NULL;
			}
			return 0;
		}
		sti();
		atomic_dec(&skb2->users);
	}

	/* Otherwise... we direct skb via semifast path. */
	skb2->pkt_type = PACKET_FASTROUTE;
	skb2->nh.raw = skb->data + ETH_HLEN;
	skb2->protocol = __constant_htons(ETH_P_IP);
	dev_fastroute_stat.deferred++;
	if (skb2 == skb)
		ei_local->skb_cache = NULL;
	*skb_p = skb2;
	return 1;

suspect:
miss:
	/* Need to read packet to memory */
	skb2 = skb;
	if (len > skb_tailroom(skb)) {
		skb2 = dev_alloc_skb(len+2);
		if (skb2 == NULL)
			goto oom;
		skb_reserve(skb2, 2);
		skb2->dev = dev;
		memcpy(skb2->data, skb->data, 60);
	}
	skb_put(skb2, len);
	if (len > 60) {
		skb2->data += 60;
		ei_block_input(dev, len - 60, skb2, (isp<<8) + 60 + 4);
		skb2->data -= 60;
	}
	if (skb2 == skb)
		ei_local->skb_cache = NULL;
	*skb_p = skb2;
	return 3;

oom:
	return 4;
}
#endif

/* We have a good packet(s), get it/them out of the buffers. */

static void ei_receive(struct device *dev, int overrun)
{
    int e8390_base = dev->base_addr;
    struct ei_device *ei_local = (struct ei_device *) dev->priv;
    unsigned char rxing_page, this_frame, next_frame;
    unsigned short current_offset;
    unsigned long start_service = jiffies;
    struct e8390_pkt_hdr rx_frame;
    int num_rx_pages = ei_local->stop_page-ei_local->rx_start_page;
    
    do {
		int pkt_len, pkt_stat, room;
		
		/* Get the rx page (incoming packet pointer). */
		outb_p(E8390_NODMA+E8390_PAGE1, e8390_base + E8390_CMD);
		rxing_page = inb_p(e8390_base + EN1_CURPAG);
		outb_p(E8390_NODMA+E8390_PAGE0, e8390_base + E8390_CMD);

		/* Remove one frame from the ring.  Boundary is always a page behind. */
		this_frame = inb_p(e8390_base + EN0_BOUNDARY) + 1;
		if (this_frame >= ei_local->stop_page) {
			if (this_frame > ei_local->stop_page)
				printk("8390.c: this_frame=%02x > stop_page=%02x\n", this_frame, ei_local->stop_page);
			this_frame = ei_local->rx_start_page;
		}

		/* Someday we'll omit the previous, iff we never get this message.
		   (There is at least one clone claimed to have a problem.)  */
		if (ei_debug > 0  &&  this_frame != ei_local->current_page)
			printk("%s: mismatched read page pointers %2x vs %2x.\n",
				   dev->name, this_frame, ei_local->current_page);
		
		room = this_frame - rxing_page;
		if (room == 0) /* Read all the frames? */
			break;				/* Done for now */

#if 0
		if (room < 0)
			room += num_rx_pages;
		if (room < 7)
			overrun = 1;
#endif

		current_offset = this_frame << 8;
		ei_get_8390_hdr(dev, &rx_frame, this_frame);

		pkt_len = rx_frame.count - sizeof(struct e8390_pkt_hdr);
		pkt_stat = rx_frame.status;
		
		/* Frame is followed by 4 byte FCS */
		next_frame = this_frame + ((pkt_len+4+4+255)>>8);
		if (next_frame >= ei_local->stop_page)
			next_frame -= num_rx_pages;

		/* Check for bogosity warned by 3c503 book: the status byte is never
		   written.  This happened a lot during testing! This code should be
		   cleaned up someday. */
		if (rx_frame.next != next_frame) {
			ei_local->current_page = rxing_page;
			outb(ei_local->current_page-1, e8390_base+EN0_BOUNDARY);
			ei_local->stat.rx_errors++;
			if (ei_debug) {
				printk("%s: frame ptrs corrupted: nf=%#2x, nxpg=%#2x c=%d tf=%#2x.\n",
					   dev->name, next_frame, rx_frame.next, rx_frame.count, this_frame);
			}
			continue;
		}

		if (overrun) {
			ei_local->stat.rx_dropped++;
			goto do_next_frame;
		}

		if (pkt_len < 60  ||  pkt_len > 1518) {
			if (ei_debug)
				printk("%s: bogus packet size: %d, status=%#2x nxpg=%#2x.\n",
					   dev->name, rx_frame.count, rx_frame.status,
					   rx_frame.next);
			ei_local->stat.rx_errors++;
			ei_local->stat.rx_length_errors++;
			goto do_next_frame;
		}

		if ((pkt_stat & 0x0F) == ENRSR_RXOK) {
			struct sk_buff *skb;

#ifdef CONFIG_NET_HW_FLOWCONTROL
			if (netdev_dropping)
				goto throttle;
#endif
#ifdef CONFIG_NET_FASTROUTE
			if (netdev_fastroute) {
				switch (e8390_fast_route(dev, &skb, pkt_len, this_frame)) {
				case 0: /* Fast route is succeed */
					goto pkt_done;
				case 1: /* Fast route is succeed, but packet is deferred */
					goto semi_done;
				case 2:	/* Fast route is hit, but packet is suspicious */
				case 3: /* Miss */
					goto process_pkt;
				case 4:
					ei_local->stat.rx_dropped++;
					overrun = 1;
					goto do_next_frame;
				default:
					panic("%s: impossible\n", dev->name);
				}
			}
#endif
			skb = dev_alloc_skb(pkt_len+2);
			if (skb == NULL) {
				if (ei_debug > 1)
					printk("%s: Couldn't allocate a sk_buff of size %d.\n",
						   dev->name, pkt_len);
				ei_local->stat.rx_dropped++;
				overrun = 1;
				goto do_next_frame;
			} else {
				skb_reserve(skb,2);	/* IP headers on 16 byte boundaries */
				skb->dev = dev;
				skb_put(skb, pkt_len);	/* Make room */
				ei_block_input(dev, pkt_len, skb, current_offset + sizeof(rx_frame));
process_pkt:
				skb->protocol=eth_type_trans(skb,dev);
semi_done:
				netif_rx(skb);
#ifdef CONFIG_NET_HW_FLOWCONTROL
				if (netdev_dropping) {
throttle:
					if (ei_local->fc_bit) {
						outb_p(E8390_RXOFF, e8390_base + EN0_RXCR);
						set_bit(ei_local->fc_bit, &netdev_fc_xoff);
					}
					overrun = 1;
				}
pkt_done:
#endif
				ei_local->stat.rx_packets++;
				ei_local->stat.rx_bytes += pkt_len;
				if (pkt_stat & ENRSR_PHY)
					ei_local->stat.multicast++;
			}
			goto do_next_frame;
		}

		if (ei_debug)
			printk("%s: bogus packet: status=%#2x nxpg=%#2x size=%d\n",
			       dev->name, rx_frame.status, rx_frame.next,
			       rx_frame.count);
		ei_local->stat.rx_errors++;
		/* NB: The NIC counts CRC, frame and missed errors. */
		if (pkt_stat & ENRSR_FO)
			ei_local->stat.rx_fifo_errors++;

do_next_frame:
		ei_local->current_page = next_frame;
		outb_p(next_frame-1, e8390_base+EN0_BOUNDARY);
    } while (jiffies - start_service < 4);

    /* We used to also ack ENISR_OVER here, but that would sometimes mask
    a real overrun, leaving the 8390 in a stopped state with rec'vr off. */
    outb_p(ENISR_RX+ENISR_RX_ERR, e8390_base+EN0_ISR);
    return;
}

/* 
 * We have a receiver overrun: we have to kick the 8390 to get it started
 * again. Problem is that you have to kick it exactly as NS prescribes in
 * the updated datasheets, or "the NIC may act in an unpredictable manner."
 * This includes causing "the NIC to defer indefinitely when it is stopped
 * on a busy network."  Ugh.
 */
static void ei_rx_overrun(struct device *dev)
{
    int e8390_base = dev->base_addr;
    unsigned long wait_start_time;
    unsigned char was_txing, must_resend = 0;
    struct ei_device *ei_local = (struct ei_device *) dev->priv;
    
    /*
     * Record whether a Tx was in progress and then issue the
     * stop command.
     */
    was_txing = inb_p(e8390_base+E8390_CMD) & E8390_TRANS;
    outb_p(E8390_NODMA+E8390_PAGE0+E8390_STOP, e8390_base+E8390_CMD);
    
    if (ei_debug > 1)
	printk("%s: Receiver overrun.\n", dev->name);
    ei_local->stat.rx_over_errors++;
    
    /* 
     * Wait a full Tx time (1.2ms) + some guard time, NS says 1.6ms total.
     * Early datasheets said to poll the reset bit, but now they say that
     * it "is not a reliable indicator and subsequently should be ignored."
     * We wait at least 10ms.
     */
    wait_start_time = jiffies;
    while (jiffies - wait_start_time <= 1*HZ/100)
	barrier();

    /*
     * Reset RBCR[01] back to zero as per magic incantation.
     */
    outb_p(0x00, e8390_base+EN0_RCNTLO);
    outb_p(0x00, e8390_base+EN0_RCNTHI);

    /*
     * See if any Tx was interrupted or not. According to NS, this
     * step is vital, and skipping it will cause no end of havoc.
     */
    if (was_txing) { 
	unsigned char tx_completed = inb_p(e8390_base+EN0_ISR) & (ENISR_TX+ENISR_TX_ERR);
	if (!tx_completed) must_resend = 1;
    }

    /*
     * Have to enter loopback mode and then restart the NIC before
     * you are allowed to slurp packets up off the ring.
     */
#ifdef CONFIG_NET_HW_FLOWCONTROL
    outb_p(E8390_RXOFF, e8390_base + EN0_RXCR);
#endif
    outb_p(E8390_TXOFF, e8390_base + EN0_TXCR);
    outb_p(E8390_NODMA + E8390_PAGE0 + E8390_START, e8390_base + E8390_CMD);

    /*
     * Clear the Rx ring of all the debris, and ack the interrupt.
     */
    ei_receive(dev, 1);
    outb_p(ENISR_OVER, e8390_base+EN0_ISR);

    /*
     * Leave loopback mode, and resend any packet that got stopped.
     */
#ifdef CONFIG_NET_HW_FLOWCONTROL
    if (!test_bit(ei_local->fc_bit, &netdev_fc_xoff))
	    outb_p(ei_local->rxcfg, e8390_base + EN0_RXCR);
#endif
    outb_p(E8390_TXCONFIG, e8390_base + EN0_TXCR); 
    if (must_resend)
    	outb_p(E8390_NODMA + E8390_PAGE0 + E8390_START + E8390_TRANS, e8390_base + E8390_CMD);
	
}

static struct net_device_stats *get_stats(struct device *dev)
{
    short ioaddr = dev->base_addr;
    struct ei_device *ei_local = (struct ei_device *) dev->priv;
    
    /* If the card is stopped, just return the present stats. */
    if (dev->start == 0) return &ei_local->stat;

    /* Read the counter registers, assuming we are in page 0. */
    ei_local->stat.rx_frame_errors += inb_p(ioaddr + EN0_COUNTER0);
    ei_local->stat.rx_crc_errors   += inb_p(ioaddr + EN0_COUNTER1);
    ei_local->stat.rx_missed_errors+= inb_p(ioaddr + EN0_COUNTER2);
    
    return &ei_local->stat;
}

/*
 * Update the given Autodin II CRC value with another data byte.
 */
static inline u32 update_crc(u8 byte, u32 current_crc)
{
	int bit;
	u8 ah = 0;

	for (bit=0; bit<8; bit++) {
		u8 carry = (current_crc>>31);
		current_crc <<= 1;
		ah = ((ah<<1) | carry) ^ byte;
		if (ah&1)
			current_crc ^= 0x04C11DB7;	/* CRC polynomial */
		ah >>= 1;
		byte >>= 1;
	}
	return current_crc;
}

/*
 * Form the 64 bit 8390 multicast table from the linked list of addresses
 * associated with this dev structure.
 */
static inline void make_mc_bits(u8 *bits, struct device *dev)
{
	struct dev_mc_list *dmi;

	for (dmi=dev->mc_list; dmi; dmi=dmi->next) {
		int i;
		u32 crc;
		if (dmi->dmi_addrlen != ETH_ALEN) {
			printk(KERN_INFO "%s: invalid multicast address length given.\n", dev->name);
			continue;
		}
		crc = 0xffffffff;	/* initial CRC value */
		for (i=0; i<ETH_ALEN; i++)
			crc = update_crc(dmi->dmi_addr[i], crc);
		/* 
		 * The 8390 uses the 6 most significant bits of the
		 * CRC to index the multicast table.
		 */
		bits[crc>>29] |= (1<<((crc>>26)&7));
	}
}

/*
 *	Set or clear the multicast filter for this adaptor.
 */
 
static void set_multicast_list(struct device *dev)
{
	short ioaddr = dev->base_addr;
	int i;
	unsigned long flags;
	struct ei_device *ei = (struct ei_device*)dev->priv;

	if (!(dev->flags&(IFF_PROMISC|IFF_ALLMULTI))) {
		memset(ei->mcfilter, 0, 8);
		if (dev->mc_list && dev->flags&IFF_MULTICAST)
			make_mc_bits(ei->mcfilter, dev);
	} else
		memset(ei->mcfilter, 0xFF, 8);	/* mcast set to accept-all */

	/* 
	 * DP8390 manuals don't specify any magic sequence for altering
	 * the multicast regs on an already running card. To be safe, we
	 * ensure multicast mode is off prior to loading up the new hash
	 * table. If this proves to be not enough, we can always resort
	 * to stopping the NIC, loading the table and then restarting.
	 */
	if (dev->start)
		outb_p(E8390_RXCONFIG, ioaddr + EN0_RXCR);
	save_flags(flags);
	cli();
	outb_p(E8390_NODMA + E8390_PAGE1, ioaddr + E8390_CMD);
	for(i = 0; i < 8; i++)
		outb_p(ei->mcfilter[i], ioaddr + EN1_MULT + i);
	outb_p(E8390_NODMA + E8390_PAGE0, ioaddr + E8390_CMD);
	restore_flags(flags);

	
	if(dev->flags&IFF_PROMISC)
		ei->rxcfg = E8390_RXCONFIG | 0x18;
	else if(dev->flags&IFF_ALLMULTI || dev->mc_list)
		ei->rxcfg = E8390_RXCONFIG | 0x08;
	else
		ei->rxcfg = E8390_RXCONFIG;
	outb_p(ei->rxcfg, ioaddr + EN0_RXCR);
}

/* Initialize the rest of the 8390 device structure. */
__initfunc(int ethdev_init(struct device *dev))
{
    if (ei_debug > 1)
		printk(version);
    
    if (dev->priv == NULL) {
		struct ei_device *ei_local;
		
		dev->priv = kmalloc(sizeof(struct ei_device), GFP_KERNEL);
		if (dev->priv == NULL)
			return -ENOMEM;
		memset(dev->priv, 0, sizeof(struct ei_device));
		ei_local = (struct ei_device *)dev->priv;
    }
    
    dev->hard_start_xmit = &ei_start_xmit;
    dev->get_stats	= get_stats;
    dev->set_multicast_list = &set_multicast_list;
#ifdef CONFIG_NET_FASTROUTE
    dev->accept_fastpath = e8390_accept_fastpath;
#endif

    ether_setup(dev);
        
    return 0;
}


/* This page of functions should be 8390 generic */
/* Follow National Semi's recommendations for initializing the "NIC". */
void NS8390_init(struct device *dev, int startp)
{
    int e8390_base = dev->base_addr;
    struct ei_device *ei_local = (struct ei_device *) dev->priv;
    int i;
    int endcfg = ei_local->word16 ? (0x48 | ENDCFG_WTS) : 0x48;
    unsigned long flags;
    
    /* Follow National Semi's recommendations for initing the DP83902. */
    outb_p(E8390_NODMA+E8390_PAGE0+E8390_STOP, e8390_base); /* 0x21 */
    outb_p(endcfg, e8390_base + EN0_DCFG);	/* 0x48 or 0x49 */
    /* Clear the remote byte count registers. */
    outb_p(0x00,  e8390_base + EN0_RCNTLO);
    outb_p(0x00,  e8390_base + EN0_RCNTHI);
    /* Set to monitor and loopback mode -- this is vital!. */
    ei_local->rxcfg = E8390_RXOFF;
    outb_p(E8390_RXOFF, e8390_base + EN0_RXCR); /* 0x20 */
    outb_p(E8390_TXOFF, e8390_base + EN0_TXCR); /* 0x02 */
    /* Set the transmit page and receive ring. */
    outb_p(ei_local->tx_start_page,	 e8390_base + EN0_TPSR);
    ei_local->tx1 = ei_local->tx2 = ei_local->tx3 = 0;
    outb_p(ei_local->rx_start_page,	 e8390_base + EN0_STARTPG);
    outb_p(ei_local->stop_page-1, e8390_base + EN0_BOUNDARY); /* 3c503 says 0x3f,NS0x26*/
    ei_local->current_page = ei_local->rx_start_page;		/* assert boundary+1 */
    outb_p(ei_local->stop_page,	  e8390_base + EN0_STOPPG);
    /* Clear the pending interrupts and mask. */
    outb_p(0xFF, e8390_base + EN0_ISR);
    ei_local->dmaing = 0;
    outb_p(0x00,  e8390_base + EN0_IMR);
    
    /* Copy the station address into the DS8390 registers. */
    save_flags(flags);
    cli();
    outb_p(E8390_NODMA + E8390_PAGE1 + E8390_STOP, e8390_base); /* 0x61 */
    for(i = 0; i < 6; i++) {
		outb_p(dev->dev_addr[i], e8390_base + EN1_PHYS + i);
    }
    
    outb_p(ei_local->rx_start_page, e8390_base + EN1_CURPAG);
    outb_p(E8390_NODMA+E8390_PAGE0+E8390_STOP, e8390_base);
    restore_flags(flags);
    dev->tbusy = 0;
    dev->interrupt = 0;
    ei_local->tx1 = ei_local->tx2 = ei_local->tx3 = 0;
    ei_local->txing = 0;
    if (startp) {
		outb_p(0xff,  e8390_base + EN0_ISR);
		ei_local->dmaing = 0;
		outb_p(ENISR_ALL,  e8390_base + EN0_IMR);
		outb_p(E8390_NODMA+E8390_PAGE0+E8390_START, e8390_base);
		outb_p(E8390_TXCONFIG, e8390_base + EN0_TXCR); /* xmit on. */
		/* 3c503 TechMan says rxconfig only after the NIC is started. */
		ei_local->rxcfg = E8390_RXCONFIG;
		outb_p(E8390_RXCONFIG,	e8390_base + EN0_RXCR); /* rx on,  */
		set_multicast_list(dev);	/* (re)load the mcast table */
    }
    return;
}

/* Trigger a transmit start, assuming the length is valid. */
static void NS8390_trigger_send(struct device *dev, unsigned int length,
								int start_page)
{
    int e8390_base = dev->base_addr;
    
    outb_p(E8390_NODMA+E8390_PAGE0, e8390_base);
    
    if (inb_p(e8390_base) & E8390_TRANS) {
		printk("%s: trigger_send() called with the transmitter busy.\n",
			   dev->name);
		return;
    }
    outb_p(length & 0xff, e8390_base + EN0_TCNTLO);
    outb_p(length >> 8, e8390_base + EN0_TCNTHI);
    outb_p(start_page, e8390_base + EN0_TPSR);
    outb_p(E8390_NODMA+E8390_TRANS+E8390_START, e8390_base);
    return;
}

#ifdef MODULE

int init_module(void)
{
     return 0;
}

void
cleanup_module(void)
{
}
#endif /* MODULE */
